package edu.upv_ehu.gbg.docencia.perfectos;

/**
 * Realiza el cálculo de números perfectos mediante 5 aproximaciones.
 * De utilidad para ver cómo el planteamiento de algoritmos adecuados es importante,
 * y para ver las posibilidades que da Java para realizar cómputos complejos.
 * 
 * Cada ejecución se limita al cálculo de una cantidad de números "razonable" para que la ejecución se realice sin una espera excesiva 
 * (20 segs la suma de los 4 primeros métodos y 2 min el quinto en un i7_870 -8 hilos- a 2.93GHz).
 * 
 * @author german
 */
public class PruebasConNumerosPerfectos {
    private PruebasConNumerosPerfectos(){}
    
    //Topes para los distintos métodos probados.
    private static final int TOPE_NAIF_Y_MEJOR=50_000; 
    private static final int MAXRO_BUENO=32; 
    private static final int MAXRO_BIGS=1300; 
    private static final int MAXRO_BIGS_PARALLEL=30_000_000;
    //Factor de certeza para el cálculo de primalidad con BigIntegers
    private static final int CERTAINTY=8;
    //Número de hilos a usar en el pool por el método paralelo
    private static final int NUMERO_DE_HILOS=Runtime.getRuntime().availableProcessors()-2;
    
    /**
     * Realiza el cálculo de números perfectos mediante 5 aproximaciones.
     * 
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        //tiempo inicial para mostrar el acumulado hasta la obtención de cada perfecto.
        //en los métodos noBIG
        long elapsedTimeControl;
        
        //Naif
        System.out.format("Probando método naif hasta %,d\n",TOPE_NAIF_Y_MEJOR);
        elapsedTimeControl=System.currentTimeMillis();
        CalculadorDiversoDeNumerosPerfectos.perfectosNaif(TOPE_NAIF_Y_MEJOR);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");

        //Mejor
        System.out.format("Probando método major hasta %,d\n",TOPE_NAIF_Y_MEJOR);
        elapsedTimeControl=System.currentTimeMillis();
        CalculadorDiversoDeNumerosPerfectos.perfectosMejor(TOPE_NAIF_Y_MEJOR);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");

        //Bueno
        System.out.format("Probando método bueno hasta la potencia %,d\n",MAXRO_BUENO);
        elapsedTimeControl=System.currentTimeMillis();
        CalculadorDiversoDeNumerosPerfectos.perfectosBueno(MAXRO_BUENO);
        elapsedTimeControl-=System.currentTimeMillis();
        System.out.println("Ejecutado en "+(-elapsedTimeControl)+" ms.----------------\n\n\n");

        //Big
        System.out.format("Probando método bueno hasta la potencia ro= %,d\n",MAXRO_BIGS);
        CalculadorDiversoDeNumerosPerfectos.perfectosBig(MAXRO_BIGS,CERTAINTY);
        System.out.println("----------------\n\n\n");

        //Threaded
        System.out.format("Probando método bueno en paralelo hasta la potencia ro= %,d\n",MAXRO_BIGS_PARALLEL);
        CalculadorDiversoDeNumerosPerfectos.perfectosBigThreaded(MAXRO_BIGS_PARALLEL,CERTAINTY,NUMERO_DE_HILOS);
        System.out.println("----------------\n\n\n");

    }

}
